"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.dkOp = exports.DkOp = void 0;
class DkOp {
    constructor(createUpdateExpressionPart) {
        this.createUpdateExpressionPart = createUpdateExpressionPart;
    }
}
exports.DkOp = DkOp;
const attributePath = ({ alias, precedingKeys }) => `${precedingKeys && precedingKeys.length > 0
    ? `${precedingKeys.map(precedingKey => `#${precedingKey}`).join('.')}.`
    : ''}#${alias}`;
exports.dkOp = {
    Value: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            return {
                updateExpression: `${attributePath({ alias, precedingKeys })} = :${alias}, `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    Add: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = ${path} + :${alias}, `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    Minus: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = ${path} - :${alias}, `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    ListAppend: (value, end = 'tail') => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = list_append(${end === 'head' ? `:${alias}, ${path}` : `${path}, :${alias}`}), `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    },
    IfNotExists: (value) => {
        const output = new DkOp(({ key, alias, precedingKeys }) => {
            const path = attributePath({ alias, precedingKeys });
            return {
                updateExpression: `${path} = if_not_exists(${path}, :${alias}), `,
                expressionAttributeNames: {
                    [`#${alias}`]: key
                },
                expressionAttributeValues: {
                    [`:${alias}`]: value
                }
            };
        });
        return output;
    }
};
